@@ -27,6 +27,14 @@
 //! Command::cargo("build").arg("-q").run_pass(OutputStr::empty(), normalize::noop());
 //! ```
 //!
+//! ### Test that `cargo miri test -q` runs successfully and has expected stdout and empty stderr without normalization
+//! ```
+//! # use tryrun::prelude::*;
+//! Command::cargo_miri("test")
+//!     .args(&["-q", "--lib", "--target-dir=target/miri"])
+//!     .run_pass(OutputStr::stdout_only("tests/ui/cargo_miri_test.stdout"), normalize::noop());
+//! ```
+//!
 //! ### Test that `rustc -V` runs successfully and has expected stdout after normalization and empty stderr
 //! ```
 //! # use tryrun::prelude::*;
@@ -106,8 +114,11 @@ use std::{
 };
 
 mod sealed {
+    use std::ffi::OsStr;
+
     pub trait Sealed {
         fn new(program: &str) -> Self;
+        fn arg(&mut self, arg: impl AsRef<OsStr>) -> &mut Self;
     }
 }
 
@@ -444,10 +455,34 @@ macro_rules! Output {
 
 /// An extension trait for [`Command`].
 pub trait CommandExt: Sealed {
-    /// Creates a [`Command`] that runs `cargo $subcommand`, with `--release` flag if `tryrun` is
-    /// built in release mode.
+    /// Creates a [`Command`] that runs `cargo $subcommand`, with flags provided by
+    /// [`Self::cargo_args`].
     #[must_use]
-    fn cargo(subcommand: impl AsRef<OsStr>) -> Self;
+    #[inline(always)]
+    fn cargo(subcommand: impl AsRef<OsStr>) -> Self
+    where
+        Self: Sized,
+    {
+        let mut cargo = Self::new(env!("CARGO"));
+        cargo.arg(subcommand).cargo_args();
+        cargo
+    }
+
+    /// Creates a [`Command`] that runs `cargo miri $subcommand`, with flags provided by
+    /// [`Self::cargo_args`].
+    #[must_use]
+    #[inline(always)]
+    fn cargo_miri(subcommand: impl AsRef<OsStr>) -> Self
+    where
+        Self: Sized,
+    {
+        let mut cargo_miri = Self::new(env!("CARGO"));
+        cargo_miri.arg("miri").arg(subcommand).cargo_args();
+        cargo_miri
+    }
+
+    /// Adds some appropriate flags to a `cargo` command.
+    fn cargo_args(&mut self) -> &mut Self;
 
     /// Adds some appropriate flags to a `rustc`-like command (such as `rustc` itself, `rustdoc`,
     /// `clippy-driver` or `miri`).
@@ -543,6 +578,11 @@ impl Sealed for Command {
     fn new(program: &str) -> Self {
         Self::new(program)
     }
+
+    #[inline(always)]
+    fn arg(&mut self, arg: impl AsRef<OsStr>) -> &mut Self {
+        self.arg(arg)
+    }
 }
 
 #[cfg(nightly)]
@@ -620,11 +660,11 @@ fn compare(expected: Option<&Path>, actual: &[u8], desc: &str) -> bool {
                 if output.stderr.is_empty() {
                     if output.stdout.is_empty() {
                         eprintln!(
-                        "error: command {} mismatched, but unfortunately a diff can't be provided; \
-                         here is the normalized actual output in bytes anyway: {:?} \
-                         (it's expected to match file {:?})",
-                        desc, actual, expected
-                    );
+                            "error: command {} mismatched, but unfortunately a diff can't be \
+                             provided; here is the normalized actual output in bytes anyway: {:?} \
+                             (it's expected to match file {:?})",
+                            desc, actual, expected
+                        );
                     } else {
                         eprintln!(
                             "error: command {} mismatched:\n{}",
@@ -666,13 +706,10 @@ fn compare(expected: Option<&Path>, actual: &[u8], desc: &str) -> bool {
 
 impl CommandExt for Command {
     #[inline(always)]
-    fn cargo(subcommand: impl AsRef<OsStr>) -> Self {
-        let mut cargo = Self::new(env!("CARGO"));
-        cargo.arg(subcommand).args(&[
-            #[cfg(release)]
-            "--release",
-        ] as &[&str]);
-        cargo
+    fn cargo_args(&mut self) -> &mut Self {
+        #[cfg(release)]
+        self.arg("--release");
+        self
     }
 
     #[inline(always)]
@@ -757,7 +794,7 @@ mod tests {
     fn compare() {
         assert!(super::compare(None, &[], ""));
         assert!(!super::compare(None, b"not empty", ""));
-        if !super::bless() {
+        if cfg!(not(miri)) && !super::bless() {
             assert!(!super::compare(Path::new(file!()).into(), b"actual", ""));
         }
     }
@@ -773,6 +810,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn cargo_release() {
         let mut cargo = Command::cargo("build");
         cargo.args(&["-q", "--release"]);
@@ -789,6 +827,26 @@ mod tests {
         }
     }
 
+    // All `cargo miri` invocations have to be placed only here, since running them in parallel
+    // may result in "Blocking waiting on file lock of ... sysroot" message.
+    #[test]
+    #[cfg_attr(miri, ignore)]
+    fn cargo_miri() {
+        let mut cargo_miri = Command::cargo_miri("test");
+        cargo_miri.args(&["-q", "--release", "--lib", "--target-dir=target/miri"]);
+        if cfg!(release) {
+            cargo_miri.run_fail(
+                OutputStr::stderr_only("tests/ui/cargo_miri_release.stderr"),
+                strip_exe(),
+            );
+        } else {
+            cargo_miri.run_pass(
+                OutputStr::stdout_only("tests/ui/cargo_miri_test.stdout"),
+                normalize::noop(),
+            );
+        }
+    }
+
     #[track_caller]
     fn assert_rustc_like_args(cmd: impl Debug, program: impl Debug) {
         assert_eq!(
@@ -805,7 +863,23 @@ mod tests {
         assert_eq!(format!("{:?}", left), format!("{:?}", right));
     }
 
+    const RELEASE: &str = if cfg!(release) { r#" "--release""# } else { "" };
+
     #[test]
+    #[cfg_attr(miri, ignore)]
+    fn cargo_args() {
+        assert_debug_eq(
+            Command::cargo(""),
+            Command::new(env!("CARGO")).arg("").cargo_args(),
+        );
+        assert_eq!(
+            format!("{:?}", Command::cargo("test")),
+            format!(r#"{:?} "test"{}"#, env!("CARGO"), RELEASE)
+        );
+    }
+
+    #[test]
+    #[cfg_attr(miri, ignore)]
     fn rustc_args() {
         assert_debug_eq(
             Command::rustc(),
@@ -824,6 +898,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn rustdoc_args() {
         assert_debug_eq(
             Command::rustdoc(),
@@ -848,6 +923,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn rustdoc_fake_empty() {
         Command::rustdoc()
             .arg("--invalid")
